@@ -2,7 +2,7 @@
 Cucumber result to Jira Xray Test repository
 
 """
-__version__ = "0.5.14"
+__version__ = "0.7.7"
 
 
 import datetime
@@ -16,8 +16,13 @@ from loguru import logger
 from collections import Counter
 
 
+class DuplicateValueError(Exception):
+    """Raised when a duplicate Scenario is found."""
+    pass
+
+
 class CuJiraX:
-    def __init__(self, jira_project: str, parent_testset_key: str = None) -> None:
+    def __init__(self, jira_project: str, parent_testset_key: str = None, addional_identifier: str = None) -> None:
         self.jira_project = jira_project
         self.jira = JiraX(jira_project)
         
@@ -26,6 +31,7 @@ class CuJiraX:
         self.testexecution_desc = None
         self.parent_testset_key = parent_testset_key
         self.result_info = result.Info(summary="TBA", description="TBA")
+        self.addional_identifier = addional_identifier
 
     def set_testsut_version(self, version: str):
         self.result_info.version = version
@@ -71,13 +77,16 @@ class CuJiraX:
             import_result=True, 
             import_testcase=True,
             ignore_duplicate=True
-        )-> dict:
+        )-> list:
 
         s1 = cucumber.Model.parse_file(cucumber_json)
-        output = {}
+        output = []
         for f in s1.__root__:
+            root = {}
             testset_name = f.uri.split("/")[-1]
             testexecution_name = self.testexecution_name or testset_name + " :: " + datetime.date.today().strftime("%Y%m%d") 
+            if self.addional_identifier:
+                testexecution_name = f"{self.addional_identifier} :: {testexecution_name}"
 
             ticket_ts, ticket_te = [
                 self.jira.create_testset(
@@ -86,10 +95,11 @@ class CuJiraX:
                 ),
                 self.testexecution or self.jira.create_testexecution(
                     summary=testexecution_name, 
-                    description=self.testexecution_desc or f.description or "TBA"
+                    description=self.testexecution_desc or f.description or "TBA",
+                    labels=[f'c{datetime.date.today().strftime("%Y%m%d")}', self.addional_identifier]
                 ),
             ]
-            output.update({
+            root.update({
                 'test_set': str(ticket_ts),
                 'testset_name': testset_name,
                 'parent_testset': self.parent_testset_key,
@@ -108,15 +118,15 @@ class CuJiraX:
                         ignore_duplicate=ignore_duplicate
                     )
                 except ValueError as e:
-                    raise type(e)(f"{f.uri}: {str(e)}") from e
+                    raise DuplicateValueError(f"{str(ticket_te)}:{f.uri}:{str(e)}")
                 
                 tests = [str(n) for n in map(
                     lambda x: self._update_description_if_exist(x,self.jira), exists)]
-                output.update({
+                root.update({
                     'existing_tests': tests,
                 })
 
-                logger.info(output)
+                logger.info(root)
                 
                 test_cases = [n for n in map(lambda x: self._new_testcase(x, self.jira_project, ticket_ts, self.parent_testset_key), new)]
                 test.bulk_import(test_cases) if test_cases else None
@@ -134,11 +144,12 @@ class CuJiraX:
                     testExecutionKey=str(ticket_te)
                 )
                 res = result.import_xray_json_results(req)
-                output.update({
+                root.update({
                     'result': _result,
                     'import_result_status': res.status_code,
                     'import_result_response': res.json()
                 })
+            output.append(root)
         return output    
         
     
@@ -156,51 +167,56 @@ class CuJiraX:
     def _get_results(cls, elements: Element, j: JiraX, ignore_duplicate):
         test_request_obj = []
         results = []
-        for el in elements:
-            test_name = cls.scenarioid_to_tescasename(el.id, el.keyword)
-            tests = j.get_tests(test_name)
-            if not tests:
-                raise ValueError("Test not created: {}".format(test_name))
-            
-            test_key = tests[0]
-            if not ignore_duplicate:
-                if len(tests) > 1:
-                    logger.error(test_name)
-                    logger.error(["{}".format(t) for t in tests])
-                    raise ValueError("More than 1 test key detected: ", tests, test_name)
-                    
-            statuses = [step.result.status.value for step in el.steps]
-            agg_result = "passed" if all(s == 'passed' for s in statuses) else "failed"
-            test_request_obj.append(result.Test(testKey=str(test_key), status=agg_result))
-            results.append(agg_result)
-        grand_result = "passed" if all(s == 'passed' for s in results) else "failed"
+        logger.info(elements)
+        if elements:
+            for el in elements:
+                test_name = cls.scenarioid_to_tescasename(el.id, el.keyword)
+                tests = j.get_tests(test_name)
+                if not tests:
+                    raise ValueError("Test not created: {}".format(test_name))
+                
+                test_key = tests[0]
+                if not ignore_duplicate:
+                    if len(tests) > 1:
+                        logger.error(test_name)
+                        logger.error(["{}".format(t) for t in tests])
+                        raise ValueError("More than 1 test key detected: ", tests, test_name)
+                        
+                statuses = [step.result.status.value for step in el.steps]
+                agg_result = "passed" if all(s == 'passed' for s in statuses) else "failed"
+                test_request_obj.append(result.Test(testKey=str(test_key), status=agg_result))
+                results.append(agg_result)
+            grand_result = "passed" if all(s == 'passed' for s in results) else "failed"
+        else:
+            grand_result = "failed"
         return test_request_obj, grand_result
 
     @classmethod
     def _split_elements_to_exist_and_new(cls, elements: Element, j: JiraX, ignore_duplicate):
         found= []
         not_found = []
-        
-        for el in elements:
-            test_name = cls.scenarioid_to_tescasename(el.id, el.keyword)
-            logger.info("searching_test_name: " + test_name)
+        logger.info(elements)
+        if elements:
+            for el in elements:
+                test_name = cls.scenarioid_to_tescasename(el.id, el.keyword)
+                logger.info("searching_test_name: " + test_name)
 
-            tests = j.get_tests(test_name)
+                tests = j.get_tests(test_name)
 
-            if tests and not ignore_duplicate:
-                if len(tests) > 1:
-                    logger.error(test_name)
-                    logger.error(["{}".format(t) for t in tests])
-                    raise ValueError("More than 1 test key detected: ", tests, test_name)
-            logger.info("found_in_jira: " + str(tests))
-            found.append(el) if j.get_tests(test_name) else not_found.append(el)
-        
-        # check duplicate for new test
-        if not ignore_duplicate:
-            new_testnames = [cls.scenarioid_to_tescasename(el.id, el.keyword) for el in not_found]
-            duplicates = [item for item, count in Counter(new_testnames).items() if count > 1]
-            if duplicates:
-                raise ValueError("More than 1 same test detected: ", duplicates)
+                if tests and not ignore_duplicate:
+                    if len(tests) > 1:
+                        logger.error(test_name)
+                        logger.error(["{}".format(t) for t in tests])
+                        raise ValueError("More than 1 test key detected: ", tests, test_name)
+                logger.info("found_in_jira: " + str(tests))
+                found.append(el) if j.get_tests(test_name) else not_found.append(el)
+            
+            # check duplicate for new test
+            if not ignore_duplicate:
+                new_testnames = [cls.scenarioid_to_tescasename(el.id, el.keyword) for el in not_found]
+                duplicates = [item for item, count in Counter(new_testnames).items() if count > 1]
+                if duplicates:
+                    raise ValueError("More than 1 same test detected: ", duplicates)
 
         return found, not_found
     
